// emonTx_Phase_Calibrator_config.ino
#include <avr/pgmspace.h>

 // Available Serial Commands
const PROGMEM char helpText1[] =                                
"\n"
"Available commands:\n"
"  l         - list the settings\n"
"  r         - restart using sketch default values\n"
"  F         - show firmware version\n"
"  ?         - show this text again\n"
"\n"
"  d<x.x>    - x.x = a floating point number for the data reporting interval in seconds\n"
"  v<x.xx>   - x.xx = a floating point number for the voltage calibration constant\n"
"  I<x>      - x = the Current input pin number. emonTx V2.2: 3,0,1, others: 1-4\n"
"  i<x.xx>   - x.xx = a floating point number for the current calibration constant\n"
"               [These calibration constants are only needed if you require to measure\n"
"                the phase difference at a known voltage and current.]\n"
"  p<x.xx>   - x.xx = a floating point number for the phase lead (degrees)\n"
"  l         - list the config values\n\n"
" PROCEDURE: \n"
" Set the emon board type and recompile & reload if necessary.\n"
" Set a Current pin, ensure \"PLL is locked\""
" Adjust phase lead (p) until \"pf:\" shows less than 0.002\n"

" WRITE DOWN the value for phase lead, repeat for each c.t. in use,\n"
" then manually transfer those to the other sketch.\n\n"
" See the documentation for full details.\n\n"
;


byte value;
 

static void list_calibration(void)
{
  Serial.println(F("Calibration:"));
  
  Serial.print(F("CT Pin = ")); Serial.println(ctPin);
  Serial.print(F("Data interval = ")); Serial.println(looptime/1000.0);
  Serial.print(F("vCal = ")); Serial.println(vCal);
  Serial.print(F("iCal = ")); Serial.println(iCal);
  Serial.print(F("phase lead = ")); Serial.println(iLead,3);
 
}


void softReset(void)
{
asm volatile ("  jmp 0");
}


void getSettings(void)
{
/*
 * Reads settings information (if available and permitted) from the serial port
 *  see the user instruction above, the comments below or the separate documentation for details
 *
 * Data is expected generally in the format
 * 
 *  [l] [x] [y] [z]
 * 
 * where:
 *  [l] = a single letter denoting the variable to adjust
 *  [x] [y] [z] etc are values to be set.
 * 
 */
 
  if (Serial.available())
  {
 
    char c = Serial.read();
    switch (c) {
      
      case 'd':
        /*  Format expected: d[x]
         * 
         * where:
         *  [x] = a floating point number for the datalogging period in s
         */
        looptime = Serial.parseFloat() * 1000;
        Serial.print(F("datalog period: ")); Serial.print(looptime/1000.0);Serial.println(F(" s"));
        break;

      case 'F': // print firmware version
        Serial.println(F(VERSION));
        break;
      
      case 'I':
        /*
         *  Format expected: I[x]
         * 
         * where:
         *  [x] = a single digit for the pin number for current
         */
        ctPin = Serial.parseInt(); 
        Serial.print(F("Current pin: "));Serial.println(ctPin);
        break;
        
      case 'i':
        /*  Format expected: i[x]
         * 
         * where:
         *  [x] = a floating point number for the Current calibration
         */
        iCal = Serial.parseFloat();
        calculateConstants();
        Serial.print(F("Current calibration: "));Serial.println(iCal);
        break;
       
      case 'l':
        list_calibration(); // print the settings & calibration values
        break;
                 
      case 'p':
        /*  Format expected: i[x]
         * 
         * where:
         *  [x] = a floating point number for the Phase calibration
         */
        iLead = Serial.parseFloat();
        Serial.print(F("Phase lead: "));Serial.println(iLead);
        break;
       
      case 'r': // restore sketch defaults
        while (Serial.available())
          Serial.read(); 
        softReset();
        break;

      case 'v':
        /*  Format expected: i[x]
         * 
         * where:
         *  [x] = a floating point number for the Voltage calibration
         */
        vCal = Serial.parseFloat();
        calculateConstants();
        Serial.print(F("Voltage calibration: "));Serial.println(vCal);
        break;
                 

      case '?':  // show Help text        
        showString(helpText1);
        Serial.println(F(" "));
        break;
      
      default:
        ;
    } //end switch
    while (Serial.available())
      Serial.read(); 

  }
}


void showHelptext(void)
{
  showString(helpText1);
}


static void showString (PGM_P s) {
  for (;;) {
    char c = pgm_read_byte(s++);
    if (c == 0)
      break;
    if (c == '\n')
      Serial.print('\r');
    Serial.print(c);
  }
}
